## $Id: batching.py 11891 2014-10-28 20:02:45Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Workflow for customers.
"""
import grok
from datetime import datetime
from zope.component import getUtility
from hurry.workflow.workflow import Transition, WorkflowState, NullCondition
from hurry.workflow.interfaces import IWorkflowState, IWorkflowTransitionEvent
from waeup.ikoba.interfaces import (
    IObjectHistory, IIkobaWorkflowInfo, IIkobaUtils,
    CREATED, ADMITTED, CLEARANCE, REQUESTED, CLEARED, PAID, RETURNING,
    REGISTERED, VALIDATED, GRADUATED, TRANSCRIPT)
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.workflow import IkobaWorkflow, IkobaWorkflowInfo
from waeup.ikoba.customers.interfaces import ICustomer, ICustomersUtils
from waeup.ikoba.utils.helpers import get_current_principal


IMPORTABLE_STATES = (ADMITTED, CLEARANCE, REQUESTED, CLEARED, PAID, RETURNING,
    REGISTERED, VALIDATED, GRADUATED)

REGISTRATION_TRANSITIONS = (
    Transition(
        transition_id = 'create',
        title = _('Create customer'),
        source = None,
        condition = NullCondition,
        msg = _('Record created'),
        destination = CREATED),

    Transition(
        transition_id = 'admit',
        title = _('Admit customer'),
        msg = _('Admitted'),
        source = CREATED,
        destination = ADMITTED),

    Transition(
        transition_id = 'reset1',
        title = _('Reset customer'),
        msg = _('Reset to initial state'),
        source = ADMITTED,
        destination = CREATED),

    Transition(
        transition_id = 'start_clearance',
        title = _('Start clearance'),
        msg = _('Clearance started'),
        source = ADMITTED,
        destination = CLEARANCE),

    Transition(
        transition_id = 'reset2',
        title = _('Reset to admitted'),
        msg = _("Reset to 'admitted'"),
        source = CLEARANCE,
        destination = ADMITTED),

    Transition(
        transition_id = 'request_clearance',
        title = _('Request clearance'),
        msg = _('Clearance requested'),
        source = CLEARANCE,
        destination = REQUESTED),

    Transition(
        transition_id = 'reset3',
        title = _('Reset to clearance started'),
        msg = _("Reset to 'clearance started'"),
        source = REQUESTED,
        destination = CLEARANCE),

    Transition(
        transition_id = 'clear',
        title = _('Clear customer'),
        msg = _('Cleared'),
        source = REQUESTED,
        destination = CLEARED),

    Transition(
        transition_id = 'reset4',
        title = _('Reset to clearance started'),
        msg = _("Reset to 'clearance started'"),
        source = CLEARED,
        destination = CLEARANCE),

    Transition(
        transition_id = 'pay_first_school_fee',
        title = _('Pay school fee'),
        msg = _('First school fee payment made'),
        source = CLEARED,
        destination = PAID),

    Transition(
        transition_id = 'approve_first_school_fee',
        title = _('Approve payment'),
        msg = _('First school fee payment approved'),
        source = CLEARED,
        destination = PAID),

    Transition(
        transition_id = 'reset5',
        title = _('Reset to cleared'),
        msg = _("Reset to 'cleared'"),
        source = PAID,
        destination = CLEARED),

    Transition(
        transition_id = 'pay_school_fee',
        title = _('Pay school fee'),
        msg = _('School fee payment made'),
        source = RETURNING,
        destination = PAID),

    Transition(
        transition_id = 'pay_pg_fee',
        title = _('Pay PG school fee'),
        msg = _('PG school fee payment made'),
        source = PAID,
        destination = PAID),

    Transition(
        transition_id = 'approve_school_fee',
        title = _('Approve school fee payment'),
        msg = _('School fee payment approved'),
        source = RETURNING,
        destination = PAID),

    Transition(
        transition_id = 'approve_pg_fee',
        title = _('Approve PG school fee payment'),
        msg = _('PG school fee payment approved'),
        source = PAID,
        destination = PAID),

    Transition(
        transition_id = 'reset6',
        title = _('Reset to returning'),
        msg = _("Reset to 'returning'"),
        source = PAID,
        destination = RETURNING),

    Transition(
        transition_id = 'register_courses',
        title = _('Register courses'),
        msg = _('Courses registered'),
        source = PAID,
        destination = REGISTERED),

    Transition(
        transition_id = 'reset7',
        title = _('Reset to school fee paid'),
        msg = _("Reset to 'school fee paid'"),
        source = REGISTERED,
        destination = PAID),

    Transition(
        transition_id = 'validate_courses',
        title = _('Validate courses'),
        msg = _('Courses validated'),
        source = REGISTERED,
        destination = VALIDATED),

    Transition(
        transition_id = 'bypass_validation',
        title = _('Return and bypass validation'),
        msg = _("Course validation bypassed"),
        source = REGISTERED,
        destination = RETURNING),

    Transition(
        transition_id = 'reset8',
        title = _('Reset to school fee paid'),
        msg = _("Reset to 'school fee paid'"),
        source = VALIDATED,
        destination = PAID),

    Transition(
        transition_id = 'return',
        title = _('Return'),
        msg = _("Returned"),
        source = VALIDATED,
        destination = RETURNING),

    Transition(
        transition_id = 'reset9',
        title = _('Reset to courses validated'),
        msg = _("Reset to 'courses validated'"),
        source = RETURNING,
        destination = VALIDATED),
    )


IMPORTABLE_TRANSITIONS = [i.transition_id for i in REGISTRATION_TRANSITIONS]

registration_workflow = IkobaWorkflow(REGISTRATION_TRANSITIONS)

class RegistrationWorkflowState(WorkflowState, grok.Adapter):
    """An adapter to adapt Customer objects to workflow states.
    """
    grok.context(ICustomer)
    grok.provides(IWorkflowState)

    state_key = 'wf.registration.state'
    state_id = 'wf.registration.id'

class RegistrationWorkflowInfo(IkobaWorkflowInfo, grok.Adapter):
    """Adapter to adapt Customer objects to workflow info objects.
    """
    grok.context(ICustomer)
    grok.provides(IIkobaWorkflowInfo)

    def __init__(self, context):
        self.context = context
        self.wf = registration_workflow

@grok.subscribe(ICustomer, IWorkflowTransitionEvent)
def handle_customer_transition_event(obj, event):
    """Append message to customer history and log file when transition happened.

    Lock and unlock clearance form.
    Trigger actions after school fee payment.
    """

    msg = event.transition.user_data['msg']
    history = IObjectHistory(obj)
    history.addMessage(msg)
    try:
        customers_container = grok.getSite()['customers']
        customers_container.logger.info('%s - %s' % (obj.customer_id,msg))
    except (TypeError, AttributeError):
        pass
    return
